library(stockassessment)
load("run/model.RData")
###############################
#
# 
#  
##############################

#  Reference points  
F_MSY <- 0.3
F_MSY_lower <- 0.18
F_MSY_upper <- 0.49
Fpa <- 0.52
Flim <- 0.73
Blim <- 14376
Bpa <- 20126
MSY_Btrigger <- 20126

# TAC & advice for 2020

prev.advice <- 0
prev.TAC <-1279

#2019 catch
cur.cat <-2264


#~~~~~~~~~~~~~~~~~~~~~~
#  FORECAST ASSUMPTIONS
#
# ---- Using the final year in the assessment (intermediate year) allows use of model estimate int yr Rec
final.yr <-max(fit$data$years)
# ---- Resample R from previous 10 years Rec
R.range <-seq(final.yr-10,final.yr-1,1)
# ---- Use final year for mean weights & L-D partition
av.yrs <-2019
#----- Use 3 year average selection patter
sel.yrs <-2017:2019
# ---- F status quo is rescaled to 2019
f_mean <-fbartable(fit)[as.character(2019),"Estimate"] 



#########  forecast   ##########

FC<-list()

#SSB target
# Stepwise % increases
SSB2021 <-3025 # Median SSB in 20201

#for (i in 1:11){

#	set.seed(12345)
#    SSBT <-SSB2021+(i-1)*SSB20201/10
#    run.lab <-paste0("2020F=Fsq then ",as.character(SSBT))
    
#	FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,NA,NA),nextssb=c(NA,3025,SSBT),year.base = final.yr,
#                               label=run.lab,rec.years=R.range,ave.years = av.yrs,
#                               overwriteSelYears = sel.yrs,splitLD = TRUE)
#}



# 2020F=Fsq then zero
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,0.0,0.0), fval=c(f_mean,NA,NA), year.base = final.yr,
                               label="2020F=Fsq then zero", rec.years=R.range, ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)

# 2020F=Fsq then Fmsy

set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,F_MSY,F_MSY), year.base = final.yr,
                               label="2020F=Fsq then Fmsy", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)

                
# 2020F=Fsq then Fmsy lower
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,F_MSY_lower,F_MSY_lower), year.base = final.yr,
                               label="2020F=Fsq then Fmsy lower",rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)


# 2020F=Fsq then Fmsy upper
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,F_MSY_upper,F_MSY_upper), year.base = final.yr,
                               label="2020F=Fsq then Fmsy upper",rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)


# 2020F=Fsq then Fpa 
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,Fpa,Fpa), year.base = final.yr,
                               label="2020F=Fsq then Fpa", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)


# 2020F=Fsq then Flim 


set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,Flim,Flim), year.base = final.yr,
                               label="2020F=Fsq then Flim", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)

# SQ
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,f_mean,f_mean),year.base = final.yr,
                               label="2020F=Fsq then Fsq", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)


# FSQ*0.79
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,0.79*f_mean,0.79*f_mean),year.base = final.yr,
                               label="2020F=Fsq then Fsq*0.79", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)

# FSQ*0.75
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,0.75*f_mean,0.75*f_mean),year.base = final.yr,
                               label="2020F=Fsq then Fsq*0.75", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)
               
# FSQ*0.5
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,0.5*f_mean,0.5*f_mean),year.base = final.yr,
                               label="2020F=Fsq then Fsq*0.5", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)
                               
# FSQ*0.25
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,0.25*f_mean,0.25*f_mean),year.base = final.yr,
                               label="2020F=Fsq then Fsq*0.25", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)   
                               
# F=0.05
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,0.05,0.05),year.base = final.yr,
                               label="2020F=Fsq then F=0.05", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)                              

#  ICES advice rule for MSY
MSYappr <- function(fit, Fmsy=NULL, Btrig=NULL, fscale_init=1, fval_init=NA, catchval_init=NA, label=NA){
  
  fscale <- fscale_init
  fval <- fval_init
  catchval <- catchval_init
  
  for (i in 2:3){
    
    fscale[i] <- NA
    fval[i] <- Fmsy
    catchval[i] <- NA
    
    set.seed(12345)
    f1 <- forecast(fit, fscale=fscale, fval=fval, catchval=catchval, ave.years=av.yrs, rec.years=R.range, overwriteSelYears=sel.yrs)
    SSB <- attr(f1,"shorttab")[3,i]
    fval[i] <- min(Fmsy*SSB/Btrig, Fmsy)
    
  }
  
  set.seed(12345)
  f_final <- forecast(fit, fscale=fscale, fval=fval, catchval=catchval, ave.years=av.yrs, rec.years=R.range, label=label, overwriteSelYears=sel.yrs, splitLD=TRUE)
  return(f_final)
}
FC[[length(FC)+1]] <- MSYappr(fit, Fmsy=F_MSY, Btrig=MSY_Btrigger, fscale_init = NA, fval_init=f_mean, catchval_init = NA, label="2020F=Fsq then Fmsy HCR")

FC[[length(FC)+1]] <- MSYappr(fit, Fmsy=F_MSY_lower, Btrig=MSY_Btrigger, fscale_init = NA, fval_init=f_mean, catchval_init = NA, label="2020F=Fsq then Fmsy HCR lower")

FC[[length(FC)+1]] <- MSYappr(fit, Fmsy=F_MSY_upper, Btrig=MSY_Btrigger, fscale_init = NA, fval_init=f_mean, catchval_init = NA, label="2020F=Fsq then Fmsy HCR upper")


#SSB target
# X% increase
set.seed(12345)
inc <-seq(0,1,0.25)
SSB2021 <-median(FC[[length(FC)]][[2]]$ssb)

i <-1
SSBtarget <-SSB2021*(1+inc[i])

FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,NA,f_mean),nextssb=c(NA,SSBtarget,NA),year.base = final.yr,
                               label=paste0("2020F=Fsq then ",(i-1)*10,"% SSB increase"),rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)


i <-2
SSBtarget <-SSB2021*(1+inc[i])
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,NA,f_mean),nextssb=c(NA,SSBtarget,NA),year.base = final.yr,
                               label=paste0("2020F=Fsq then ",(i-1)*25,"% SSB increase"),rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)


i <-3
SSBtarget <-SSB2021*(1+inc[i])
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,NA,f_mean),nextssb=c(NA,SSBtarget,NA),year.base = final.yr,
                               label=paste0("2020F=Fsq then ",(i-1)*25,"% SSB increase"),rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)

i <-4
SSBtarget <-SSB2021*(1+inc[i])
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,NA,f_mean),nextssb=c(NA,SSBtarget,NA),year.base = final.yr,
                               label=paste0("2020F=Fsq then ",(i-1)*25,"% SSB increase"),rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)

i <-5
SSBtarget <-SSB2021*(1+inc[i])
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fscale=c(NA,NA,NA), fval=c(f_mean,NA,f_mean),nextssb=c(NA,SSBtarget,NA),year.base = final.yr,
                               label=paste0("2020F=Fsq then ",(i-1)*25,"% SSB increase"),rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)
                               
# 2020F=Fsq then catch=prev.TAC
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fval=c(f_mean,NA,NA), catchval=c(NA,prev.TAC,prev.TAC), year.base = final.yr,
                               label="2020F=Fsq then prev.TAC", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)


# 2020F=Fsq then catch=prev.TAC*0.75
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fval=c(f_mean,NA,NA), catchval=c(NA,prev.TAC*0.75,prev.TAC*0.75), year.base = final.yr,
                               label="2020F=Fsq then prev.TAC*0.75", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)

# 2020F=Fsq then catch=prev.TAC*0.5
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fval=c(f_mean,NA,NA), catchval=c(NA,prev.TAC*0.5,prev.TAC*0.5), year.base = final.yr,
                               label="2020F=Fsq then prev.TAC*0.5", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)
                               
# 2020F=Fsq then catch=prev.TAC*0.25
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fval=c(f_mean,NA,NA), catchval=c(NA,prev.TAC*0.25,prev.TAC*0.25), year.base = final.yr,
                               label="2020F=Fsq then prev.TAC*0.25", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)

# 2020F=Fsq then catch=2019catch
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fval=c(f_mean,NA,NA), catchval=c(NA,cur.cat,cur.cat), year.base = final.yr,
                               label="2020F=Fsq then 2019catch", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)


# 2020F=Fsq then catch=2019catch*0.75
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fval=c(f_mean,NA,NA), catchval=c(NA,cur.cat*0.75,cur.cat*0.75), year.base = final.yr,
                               label="2020F=Fsq then 2019catch*0.75", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)

# 2020F=Fsq then catch=2019catch*0.5
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fval=c(f_mean,NA,NA), catchval=c(NA,cur.cat*0.5,cur.cat*0.5), year.base = final.yr,
                               label="2020F=Fsq then 2019catch*0.5", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)
                               
# 2020F=Fsq then catch=2019catch*0.25
set.seed(12345)
FC[[length(FC)+1]] <- forecast(fit, fval=c(f_mean,NA,NA), catchval=c(NA,cur.cat*0.25,cur.cat*0.25), year.base = final.yr,
                               label="2020F=Fsq then 2019catch*0.25", rec.years=R.range,ave.years = av.yrs,
                               overwriteSelYears = sel.yrs,splitLD = TRUE)
                               
                               


save(FC, file="run/forecast.RData")
